package pack.listeners;

import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import pack.Account;
import pack.Item;
import pack.LoadedUser;
import pack.User;
import pack.WarnMessage;
import pack.util.Database;
import pack.util.Message;
import pack.util.Values;

@ManagedBean
@SessionScoped
public class Listeners {
	
	@ManagedProperty("#{values}")
	private Values values;
	
	@ManagedProperty("#{database}")
	private Database database;
	
	@ManagedProperty("#{message}")
	private Message message;
	
	@ManagedProperty("#{item}")
	private Item item;
	
	@ManagedProperty("#{user}")
	private User user;
	
	@ManagedProperty("#{account}")
	private Account account;
	
	@ManagedProperty("#{loadedUser}")
	private LoadedUser loadedUser;
	
	
	public void plusValueItem(String id, String name) {
		if(database.addValueToItem(values.getInputValueItem(), id)){
			database.addMessageToHistory(id, name, user.getId(), user.getFirstname() + " " + user.getLastname(), "Prijato", "Prijaté množství: " + values.getInputValueItem());
			message.addMessage(FacesMessage.SEVERITY_INFO, "Přidáno");
		}
		values.setInputValueItem("");
		updateListOfItems();
	}
	
	public void minusValueItem(String id, String name){
		if(database.removeValueToItem(values.getInputValueItem(), id)){
			database.addMessageToHistory(id, name, user.getId(), user.getFirstname() + " " + user.getLastname(), "Vydáno", "Vydané množství: " + values.getInputValueItem());
			message.addMessage(FacesMessage.SEVERITY_INFO, "Odebráno");
			updateWarnMessage();
		}
		values.setInputValueItem("");
		updateListOfItems();
	}
	
	public void saveItem(){
		if(Integer.parseInt(item.getQuantity()) > Integer.parseInt(item.getCapacity())){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba množství je větší než kapacita");
			return;
		}
		if(Integer.parseInt(item.getBorder()) > Integer.parseInt(item.getCapacity())){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba min hranice je větší než kapacita");
			return;
		}
		String historyMessage = "ID: " + item.getId() + ", Název: " + item.getName() + ", Množství: " + item.getQuantity() + ", Hlídané min: " + item.getBorder() + ", Max kapacita: " + item.getCapacity();
		
		if(values.isNewItem()==true){
			if(!database.addNewItem(item)){
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba položka nepřidána");
				return;
			}else {
				database.addMessageToHistory(item.getId(), item.getName(), user.getId(), user.getFirstname() + " " + user.getLastname(), "Vytvorena položka", historyMessage);
				message.addMessage(FacesMessage.SEVERITY_INFO, "Položka přidána");
				updateWarnMessage();
			}
			values.setNewItem(false);
		}else {
			if(!database.updateItem(item)){
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba položka neuložena");
			}else {
				database.addMessageToHistory(item.getId(), item.getName(), user.getId(), user.getFirstname() + " " + user.getLastname(), "Zmena položky", historyMessage);
				message.addMessage(FacesMessage.SEVERITY_INFO, "Položka uložena");
				updateWarnMessage();
			}
		}
	}
	
	public void saveLoadedUser(){
		
		if(values.isNewUser()==true){
			if(!database.addNewUser(loadedUser)){
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba uživatel nepřidán");
			}else {
				message.addMessage(FacesMessage.SEVERITY_INFO, "Uživatel přidán");
				values.setNewUser(false);
			}
		}else {
			if(!database.updateUser(loadedUser)){
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba uživatel neuložen");
			}else {
				database.loadLoggedUser(user.getId());
				message.addMessage(FacesMessage.SEVERITY_INFO, "Uživatel uložen");
			}
		}
	}
	
	public void saveUser(){
		
		if(!database.updateUser(user, account)){
			database.loadLoggedUser(user.getId());
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba uživatel neuložen");
		}else {
			message.addMessage(FacesMessage.SEVERITY_INFO, "Uživatel uložen");
		}
	}
	
	
	
	public void removeItem(){
		if(!database.removeItem(item.getId(), item.getIdOfPhoto())){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba položka nesmazána");
			return;
		}
		database.addMessageToHistory(item.getId(), item.getName(), user.getId(), user.getFirstname() + " " + user.getLastname(), "Smazána položka", "");
		message.addMessage(FacesMessage.SEVERITY_INFO, "Položka smazána");
		item.setDefault();
		account.setPage("repository");
		updateListOfItems();
		updateWarnMessage();
	}
	
	public void removeUser(){
		if(!database.removeUser(loadedUser.getId(), loadedUser.getPhotoId())){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba položka nesmazána");
			return;
		}
		message.addMessage(FacesMessage.SEVERITY_INFO, "Položka smazána");
		loadedUser.setDefault();
		account.setPage("users");
		updateListOfUsers();
	}
	
	public void deleteHistoryMessages(){
		database.deleteHistoryMessage();
		if(!database.addMessageToHistory("0", "", user.getId(), user.getFirstname() + " " + user.getLastname(), "Smazána historie", "")){
		}
		message.addMessage(FacesMessage.SEVERITY_INFO, "Historie smazána");
		updateListOfHistory();
	}
	
	public void loadItem(String id) {
		if(!database.loadItem(id)){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba ID nenalezeno");
			return;
		}
		account.setPage("item");
		values.setNewItem(false);
	}
	
	public void loadUser(String id) {
		if(!database.loadUser(id)){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "Chyba ID nenalezeno");
			return;
		}
		account.setPage("loadedUser");
		values.setNewUser(false);
	}
	
	public void newItem() {
		item.setDefault();
		item.setId(database.nextIdOfItem());
		account.setPage("item");
		values.setNewItem(true);
	}
	
	public void newUser() {
		loadedUser.setDefault();
		loadedUser.setId(database.nextIdOfUser());
		account.setPage("loadedUser");
		values.setNewUser(true);
	}
	
	public void updateCheckedWarnMessage() {
		List<WarnMessage> listOfSelectedMessage = values.getSelectedList();
		StringBuilder historyMessage = new StringBuilder("ID: ");
		if(listOfSelectedMessage==null){
			return;
		}
		for(WarnMessage m : listOfSelectedMessage){
			historyMessage.append(m.getId() + ", ");
		}
		if(!database.updateCheckedWarnMessage(listOfSelectedMessage)){
			message.addMessage(FacesMessage.SEVERITY_ERROR, "SQL operace nezdařena rollback");
			return;
		}
		database.addMessageToHistory("0", "", user.getId(), user.getFirstname() + " " + user.getLastname(), "Smazána varovná zpráva", historyMessage.toString());
		updateWarnMessage();
		values.setListOfWarnMessage(database.listOfWarnMessage());
		message.addMessage(FacesMessage.SEVERITY_INFO, "Smazáno");
	}
	
	public void updateListOfItems(){
		values.setListOfItems(database.listOfItems(values.getSearchedRepositoryString()));
	}
	
	public void updateListOfUsers(){
		values.setListOfUsers(database.listOfUsers(values.getSearchedUsersString()));
	}
	
	public void updateListOfWarnMessage(){
		values.setListOfWarnMessage(database.listOfWarnMessage());
	}
	
	public void updateWarnMessage(){
		values.setWarnMessage(database.warnMessage());
	}
	
	public void updateListOfHistory(){
		values.setListOfHistoryMessage(database.listOfHistoryMessage(values.getSearchedHistoryString()));
	}
	
	public void toRepository(){
		item.setDefault();
		account.setPage("repository");
		updateListOfItems();
	}
	
	public void toUsers(){
		loadedUser.setDefault();
		account.setPage("users");
		updateListOfUsers();
	}
	
	public void toWarning(){
		account.setPage("warning");
		updateListOfWarnMessage();
	}
	
	public void toHistory(){
		account.setPage("history");
		updateListOfHistory();
	}
	
	public void toSetting(){
		account.setPage("setting");
	}
	
	//--------------------------setters/getters-------------------------------------------------

	public Values getValues() {
		return values;
	}

	public void setValues(Values values) {
		this.values = values;
	}

	public Database getDatabase() {
		return database;
	}

	public void setDatabase(Database database) {
		this.database = database;
	}

	public Message getMessage() {
		return message;
	}

	public void setMessage(Message message) {
		this.message = message;
	}

	public Item getItem() {
		return item;
	}

	public void setItem(Item item) {
		this.item = item;
	}

	public Account getAccount() {
		return account;
	}

	public void setAccount(Account account) {
		this.account = account;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public LoadedUser getLoadedUser() {
		return loadedUser;
	}

	public void setLoadedUser(LoadedUser loadedUser) {
		this.loadedUser = loadedUser;
	}
}
