package pack.util;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import pack.Account;
import pack.HistoryMessage;
import pack.Item;
import pack.LoadedUser;
import pack.Login;
import pack.User;
import pack.WarnMessage;

@ManagedBean(eager=true)
@SessionScoped
public class Database {
	public static final String USERNAME = Config.getDB_USERNAME();
	public static final String PASSWORD = Config.getDB_PASSWORD();
	public static final String HOST = Config.getDB_HOST();
	public static final String PORT = Config.getDB_PORT();
	public static final String DATABASE_NAME = Config.getDB_NAME();
	public static final String url = "jdbc:mysql://" + HOST + ":" + PORT + "/" + DATABASE_NAME;

	@ManagedProperty("#{user}")
	private User user;

	@ManagedProperty("#{account}")
	private Account account;
	
	@ManagedProperty("#{message}")
	private Message message;
	
	@ManagedProperty("#{item}")
	private Item item;
	
	@ManagedProperty("#{loadedUser}")
	private LoadedUser loadedUser;

	//-------------------------methods-------------------------------------------------
	
	public boolean login(Login login) {
		String sql = "SELECT * FROM account INNER JOIN user ON account.idu=user.idu WHERE user.idu=? AND user.password=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, login.getId());
			ps.setString(2, login.getPassword());
			res = ps.executeQuery();
			while (res.next()) {
				user.setId(res.getString("idu"));
				user.setPass(res.getString("password"));
				user.setFirstname(res.getString("firstname"));
				user.setLastname(res.getString("lastname"));
				user.setSex(res.getString("sex"));
				user.setEmail(res.getString("email"));
				user.setPhoneNumber(res.getString("phoneNumber"));
				user.setAdress(res.getString("adress"));
				user.setProfession(res.getString("profession"));
				user.setWorkplace(res.getString("workplace"));
				user.setPhotoId(res.getString("photoId"));
				account.setPage("application");
				account.setCssStyle(res.getString("cssStyle"));
				account.setLogged(true);
				state = account.isLogged();
			}
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean loadUser(String id){
		String sql = "SELECT * FROM user WHERE idu=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, id);
			res = ps.executeQuery();
			while (res.next()) {
				loadedUser.setId(res.getString("idu"));
				loadedUser.setPass(res.getString("password"));
				loadedUser.setFirstname(res.getString("firstname"));
				loadedUser.setLastname(res.getString("lastname"));
				loadedUser.setSex(res.getString("sex"));
				loadedUser.setEmail(res.getString("email"));
				loadedUser.setPhoneNumber(res.getString("phoneNumber"));
				loadedUser.setAdress(res.getString("adress"));
				loadedUser.setProfession(res.getString("profession"));
				loadedUser.setWorkplace(res.getString("workplace"));
				loadedUser.setPhotoId(res.getString("photoId"));
				state = true;
			}
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean loadLoggedUser(String id){
		String sql = "SELECT * FROM account INNER JOIN user ON account.idu=user.idu WHERE user.idu=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, id);
			res = ps.executeQuery();
			while (res.next()) {
				user.setId(res.getString("idu"));
				user.setPass(res.getString("password"));
				user.setFirstname(res.getString("firstname"));
				user.setLastname(res.getString("lastname"));
				user.setSex(res.getString("sex"));
				user.setEmail(res.getString("email"));
				user.setPhoneNumber(res.getString("phoneNumber"));
				user.setAdress(res.getString("adress"));
				user.setProfession(res.getString("profession"));
				user.setWorkplace(res.getString("workplace"));
				user.setPhotoId(res.getString("photoId"));
				account.setCssStyle(res.getString("cssStyle"));
				state = true;
			}
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public List<Item> listOfItems(String searchedRepositoryString) {
		String sql = "SELECT * FROM item";
		String sql2 = "SELECT * FROM item WHERE id=?";
		String sql3 = "SELECT * FROM item WHERE name=?";
		boolean isName = false;
		List<Item> items = new ArrayList<Item>();
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		ResultSet res = null;

		if(searchedRepositoryString==null){
			searchedRepositoryString = "";
		}
		
		char[] charArray = searchedRepositoryString.toCharArray();
		for(char a : charArray){
			if(!Character.isDigit(a)){
				isName = true;
			}
		}
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			if(isName){
				ps = c.prepareStatement(sql3);
			}else {
				ps = c.prepareStatement(sql2);
			}
			ps.setString(1, searchedRepositoryString);
			if(searchedRepositoryString.length()<1){
				res = stmt.executeQuery(sql);
			}else {
				res = ps.executeQuery();
			}
			while (res.next()) {
				Item item = new Item();
				item.setId(res.getString("id"));
				item.setName(res.getString("name"));
				item.setQuantity(res.getString("quantity"));
				item.setBorder(res.getString("border"));
				item.setCapacity(res.getString("capacity"));
				item.setIdOfPhoto(res.getString("photoId"));
				items.add(item);
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return items;
	}
	
	public List<User> listOfUsers(String searchedUsersString) {
		String sql = "SELECT * FROM user";
		String sql2 = "SELECT * FROM user WHERE idu=?";
		String sql3 = "SELECT * FROM user WHERE firstname=? OR lastname=? OR profession=? OR workplace=?";
		boolean isName = false;
		List<User> users = new ArrayList<>();
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		ResultSet res = null;

		if(searchedUsersString==null){
			searchedUsersString = "";
		}
		
		char[] charArray = searchedUsersString.toCharArray();
		for(char a : charArray){
			if(!Character.isDigit(a)){
				isName = true;
			}
		}
		String[] words = searchedUsersString.split(" ");
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			if(isName){
				ps = c.prepareStatement(sql3);
				if(words.length>1){
					ps.setString(1, words[0]);
					ps.setString(2, words[1]);
					ps.setString(3, words[1]);
					ps.setString(4, words[1]);
				}else {
					ps.setString(1, words[0]);
					ps.setString(2, words[0]);
					ps.setString(3, words[0]);
					ps.setString(4, words[0]);
				}
			}else {
				ps = c.prepareStatement(sql2);
				ps.setString(1, searchedUsersString);
			}
			
			if(searchedUsersString.length()<1){
				res = stmt.executeQuery(sql);
			}else {
				res = ps.executeQuery();
			}
			while (res.next()) {
				User user = new User();
				user.setId(res.getString("idu"));
				user.setPass(res.getString("password"));
				user.setFirstname(res.getString("firstname"));
				user.setLastname(res.getString("lastname"));
				user.setSex(res.getString("sex"));
				user.setEmail(res.getString("email"));
				user.setPhoneNumber(res.getString("phoneNumber"));
				user.setAdress(res.getString("adress"));
				user.setProfession(res.getString("profession"));
				user.setWorkplace(res.getString("workplace"));
				user.setPhotoId(res.getString("photoId"));
				users.add(user);
			}
			res.close();
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return users;
	}
	
	public boolean addValueToItem(String val, String id) {
		String sql = "SELECT quantity,capacity FROM item WHERE id=?";
		boolean state = false;
		int capacity = 0;
		int quantity = 0;
		int value = Integer.parseInt(val);
		String sql2 = "UPDATE item SET quantity=quantity+" + value + " WHERE id=?";
		Connection c = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, id);
			res = ps.executeQuery();
			while (res.next()) {
				capacity = res.getInt("capacity");
				quantity = res.getInt("quantity");
			}
			if((capacity-(value+quantity)) > 0){
				ps2 = c.prepareStatement(sql2);
				ps2.setString(1, id);
				if(ps2.executeUpdate()==1){
					state = true;
				}
			}else {
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Překročení kapacity položky");
			}
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean removeValueToItem(String val, String id) {
		String sql = "SELECT quantity FROM item WHERE id=?";
		boolean state = false;
		int quantity = 0;
		int value = Integer.parseInt(val);
		String sql2 = "UPDATE item SET quantity=quantity-" + value + " WHERE id=?";
		Connection c = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, id);
			res = ps.executeQuery();
			while (res.next()) {
				quantity = res.getInt("quantity");
			}
			if((quantity-value) > 0){
				ps2 = c.prepareStatement(sql2);
				ps2.setString(1, id);
				if(ps2.executeUpdate()==1){
					state = true;
					checkBorder(id);
				}
			}else {
				message.addMessage(FacesMessage.SEVERITY_ERROR, "Nedostatečné množství na skladě");
			}
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean loadItem(String id) {
		String sql = "SELECT * FROM item WHERE id=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setString(1, id);
			res = ps.executeQuery();
			while (res.next()) {
				item.setId(res.getString("id"));
				item.setName(res.getString("name"));
				item.setQuantity(res.getString("quantity"));
				item.setBorder(res.getString("border"));
				item.setCapacity(res.getString("capacity"));
				item.setIdOfPhoto(res.getString("photoId"));
				state = true;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public String nextIdOfItem() {
		String sql = "SHOW TABLE STATUS WHERE `Name` =  'item'";
		Connection c = null;
		Statement stmt = null;
		ResultSet res = null;
		String nextId = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			res = stmt.executeQuery(sql);
			while (res.next()) {
				nextId = res.getString("Auto_increment");
			}
			if(nextId==null){
				throw new SQLException();
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return nextId;
	}
	
	public String nextIdOfUser() {
		String sql = "SHOW TABLE STATUS WHERE `Name` =  'user'";
		Connection c = null;
		Statement stmt = null;
		ResultSet res = null;
		String nextId = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			res = stmt.executeQuery(sql);
			while (res.next()) {
				nextId = res.getString("Auto_increment");
			}
			if(nextId==null){
				throw new SQLException();
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return nextId;
	}
	
	public boolean addNewItem(Item it) {
		String sql = "INSERT INTO item(id,name,quantity,border,capacity,photoId) VALUES(null,?,?,?,?,?)";
		String sql2 = "SHOW TABLE STATUS WHERE `Name` =  'itemPhoto'";
		String sql3 = "SELECT id FROM item WHERE id=?";
		String sql4 = "INSERT INTO itemPhoto(id,photo) VALUES (null, ?)";
		boolean state = false;
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		ResultSet res = null;
		ResultSet res2 = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			stmt = c.createStatement();
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql3);
			ps3 = c.prepareStatement(sql4);
			
			ps2.setString(1, it.getId());
			res = ps2.executeQuery();
			if(res.next()){
				message.addMessage(FacesMessage.SEVERITY_WARN, "Id " + it.getId() +" je už obsazené, přiděleno nové Id, uložte znovu");
				it.setId(nextIdOfItem());
				res.close();
				return false;
			}
			res.close();
			
			if(it.getFileToUpload().getSize()!=0){
				res2 = stmt.executeQuery(sql2);
				res2.next();
				String photoId = res2.getString("Auto_increment");
				res2.close();
				
				ps3.setBinaryStream(1, it.getFileToUpload().getInputstream());
				if(ps3.executeUpdate()!=1){
					c.rollback();
					return false;
				}
				
				ps.setString(5, photoId);
				
				it.setFileToUpload(null);
				it.setIdOfPhoto(photoId);
			}else {
				ps.setString(5, null);
				it.setIdOfPhoto(null);
			}
			
			ps.setString(1, it.getName());
			ps.setString(2, it.getQuantity());
			ps.setString(3, it.getBorder());
			ps.setString(4, it.getCapacity());
			
			if(ps.executeUpdate()==1){
				state = true;
				c.commit();
				checkBorder(it.getId());
			}else {
				c.rollback();
				return false;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} catch (IOException e) {
			e.getMessage();
		} finally {
			try {
				
				ps3.close();
				ps2.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean addNewUser(LoadedUser u) {
		String sql = "INSERT INTO user(idu,password,sex,firstname,lastname,email,phoneNumber,adress,profession,workplace,photoId) VALUES(null,?,?,?,?,?,?,?,?,?,?)";
		String sql2 = "SHOW TABLE STATUS WHERE `Name` =  'userPhoto'";
		String sql3 = "SELECT idu FROM user WHERE idu=?";
		String sql4 = "INSERT INTO userPhoto(id,photo) VALUES (null, ?)";
		String sql5 = "SHOW TABLE STATUS WHERE `Name` =  'user'";
		String sql6 = "INSERT INTO account(ida,idu) VALUES(null,?)";
		boolean state = false;
		Connection c = null;
		Statement stmt = null;
		Statement stmt2 = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		PreparedStatement ps4 = null;
		ResultSet res = null;
		ResultSet res2 = null;
		ResultSet res3 = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			stmt = c.createStatement();
			stmt2 = c.createStatement();
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql3);
			ps3 = c.prepareStatement(sql4);
			ps4 = c.prepareStatement(sql6);
			
			
			ps2.setString(1, u.getId());
			res = ps2.executeQuery();
			if(res.next()){
				message.addMessage(FacesMessage.SEVERITY_WARN, "Id " + u.getId() +" je už obsazené, přiděleno nové Id, uložte znovu");
				u.setId(nextIdOfUser());
				res.close();
				return false;
			}
			res.close();
			
			if(u.getFileToUpload().getSize()!=0){
				res2 = stmt.executeQuery(sql2);
				res2.next();
				String photoId = res2.getString("Auto_increment");
				res2.close();
				
				ps3.setBinaryStream(1, u.getFileToUpload().getInputstream());
				if(ps3.executeUpdate()!=1){
					c.rollback();
					return false;
				}
				
				ps.setString(10, photoId);
				
				u.setFileToUpload(null);
				u.setPhotoId(photoId);
			}else {
				ps.setString(10, null);
				u.setPhotoId(null);
			}
			
			ps.setString(1, u.getPass());
			ps.setString(2, u.getSex());
			ps.setString(3, u.getFirstname());
			ps.setString(4, u.getLastname());
			ps.setString(5, u.getEmail());
			ps.setString(6, u.getPhoneNumber());
			ps.setString(7, u.getAdress());
			ps.setString(8, u.getProfession());
			ps.setString(9, u.getWorkplace());
			
			res3 = stmt2.executeQuery(sql5);
			
			if(ps.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			if(!res3.next()){
				c.rollback();
				res3.close();
				return false;
			}
			
			ps4.setString(1, res3.getString("Auto_increment"));
			res3.close();
			
			if(ps4.executeUpdate()==1){
				state = true;
				c.commit();
			}else {
				c.rollback();
				return false;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} catch (IOException e) {
			e.getMessage();
		} finally {
			try {
				ps4.close();
				ps3.close();
				ps2.close();
				ps.close();
				stmt2.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean updateItem(Item it) {
		String sql = "UPDATE item SET name=?, quantity=?, border=?, capacity=?, photoId=? WHERE id=?";
		String sql2 = "UPDATE itemPhoto SET photo=? WHERE id=?";
		String sql3 = "SHOW TABLE STATUS WHERE `Name` = 'itemPhoto'";
		String sql4 = "INSERT INTO itemPhoto(id,photo) VALUES(null,?)";
		boolean state = false;
		String idOfPhoto = null;
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			stmt = c.createStatement();
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql2);
			ps3 = c.prepareStatement(sql4);
			
			idOfPhoto = it.getIdOfPhoto();
			
			if(it.getFileToUpload()!=null){
				if(it.getFileToUpload().getSize()!=0){
					if(idOfPhoto==null){
						res = stmt.executeQuery(sql3);
						res.next();
						idOfPhoto = res.getString("Auto_increment");
						res.close();
						ps3.setBinaryStream(1, it.getFileToUpload().getInputstream());
						if(ps3.executeUpdate()!=1){
							c.rollback();
							return false;
						}
						it.setIdOfPhoto(idOfPhoto);
					}else {
						ps2.setBinaryStream(1, it.getFileToUpload().getInputstream());
						ps2.setString(2, idOfPhoto);
						if(ps2.executeUpdate()!=1){
							c.rollback();
							return false;
						}
					}
					it.setFileToUpload(null);
				}
			}
			
			ps.setString(1, it.getName());
			ps.setString(2, it.getQuantity());
			ps.setString(3, it.getBorder());
			ps.setString(4, it.getCapacity());
			ps.setString(5, idOfPhoto);
			ps.setString(6, it.getId());
			
			if(ps.executeUpdate()==1){
				state = true;
				c.commit();
				checkBorder(it.getId());
			}else {
				c.rollback();
				return false;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} catch (IOException e) {
			e.getMessage();
		} finally {
			try {
				ps3.close();
				ps2.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean updateUser(User u, Account a) {
		String sql = "UPDATE user SET firstname=?, lastname=?, password=?, sex=?, workplace=?, profession=?, email=?, phoneNumber=?, adress=?, photoId=? WHERE idu=?";
		String sql2 = "UPDATE userPhoto SET photo=? WHERE id=?";
		String sql3 = "SHOW TABLE STATUS WHERE `Name` = 'userPhoto'";
		String sql4 = "INSERT INTO userPhoto(id,photo) VALUES(null,?)";
		String sql5 = "UPDATE account SET cssStyle=? WHERE idu=?";
		boolean state = false;
		String idOfPhoto = null;
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		PreparedStatement ps4 = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			stmt = c.createStatement();
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql2);
			ps3 = c.prepareStatement(sql4);
			ps4 = c.prepareStatement(sql5);
			
			idOfPhoto = u.getPhotoId();
			if(u.getFileToUpload()!=null){
				if(u.getFileToUpload().getSize()!=0){
					if(idOfPhoto==null){
						res = stmt.executeQuery(sql3);
						res.next();
						idOfPhoto = res.getString("Auto_increment");
						res.close();
						ps3.setBinaryStream(1, u.getFileToUpload().getInputstream());
						if(ps3.executeUpdate()!=1){
							c.rollback();
							return false;
						}
						u.setPhotoId(idOfPhoto);
					}else {
						ps2.setBinaryStream(1, u.getFileToUpload().getInputstream());
						ps2.setString(2, idOfPhoto);
						if(ps2.executeUpdate()!=1){
							c.rollback();
							return false;
						}
					}
					u.setFileToUpload(null);
				}
			}
			
			ps.setString(1, u.getFirstname());
			ps.setString(2, u.getLastname());
			ps.setString(3, u.getPass());
			ps.setString(4, u.getSex());
			ps.setString(5, u.getWorkplace());
			ps.setString(6, u.getProfession());
			ps.setString(7, u.getEmail());
			ps.setString(8, u.getPhoneNumber());
			ps.setString(9, u.getAdress());
			ps.setString(10, idOfPhoto);
			ps.setString(11, u.getId());
			
			if(ps.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			ps4.setString(1, a.getCssStyle());
			ps4.setString(2, u.getId());
			
			if(ps4.executeUpdate()==1){
				state = true;
				c.commit();
			}else {
				c.rollback();
				return false;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} catch (IOException e) {
			e.getMessage();
		} finally {
			try {
				ps3.close();
				ps2.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean updateUser(LoadedUser u) {
		String sql = "UPDATE user SET firstname=?, lastname=?, password=?, sex=?, workplace=?, profession=?, email=?, phoneNumber=?, adress=?, photoId=? WHERE idu=?";
		String sql2 = "UPDATE userPhoto SET photo=? WHERE id=?";
		String sql3 = "SHOW TABLE STATUS WHERE `Name` = 'userPhoto'";
		String sql4 = "INSERT INTO userPhoto(id,photo) VALUES(null,?)";
		boolean state = false;
		String idOfPhoto = null;
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			stmt = c.createStatement();
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql2);
			ps3 = c.prepareStatement(sql4);
			
			idOfPhoto = u.getPhotoId();
			if(u.getFileToUpload()!=null){
				if(u.getFileToUpload().getSize()!=0){
					if(idOfPhoto==null){
						res = stmt.executeQuery(sql3);
						res.next();
						idOfPhoto = res.getString("Auto_increment");
						res.close();
						ps3.setBinaryStream(1, u.getFileToUpload().getInputstream());
						if(ps3.executeUpdate()!=1){
							c.rollback();
							return false;
						}
						u.setPhotoId(idOfPhoto);
					}else {
						ps2.setBinaryStream(1, u.getFileToUpload().getInputstream());
						ps2.setString(2, idOfPhoto);
						if(ps2.executeUpdate()!=1){
							c.rollback();
							return false;
						}
					}
					u.setFileToUpload(null);
				}
			}
			
			ps.setString(1, u.getFirstname());
			ps.setString(2, u.getLastname());
			ps.setString(3, u.getPass());
			ps.setString(4, u.getSex());
			ps.setString(5, u.getWorkplace());
			ps.setString(6, u.getProfession());
			ps.setString(7, u.getEmail());
			ps.setString(8, u.getPhoneNumber());
			ps.setString(9, u.getAdress());
			ps.setString(10, idOfPhoto);
			ps.setString(11, u.getId());
			
			if(ps.executeUpdate()==1){
				state = true;
				c.commit();
			}else {
				c.rollback();
				return false;
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} catch (IOException e) {
			e.getMessage();
		} finally {
			try {
				ps3.close();
				ps2.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean removeItem(String id, String photoId) {
		String sql = "DELETE FROM item WHERE id=?";
		String sql2 = "DELETE FROM warning WHERE idi=?";
		String sql3 = "DELETE FROM itemPhoto WHERE id=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;

		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			ps = c.prepareStatement(sql);
			ps2= c.prepareStatement(sql2);
			ps3= c.prepareStatement(sql3);
			ps.setString(1, id);
			ps2.setString(1, id);
			
			if(ps2.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			if(ps.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			if(photoId!=null){
				ps3.setString(1, photoId);
				if(ps3.executeUpdate()==1){
					state = true;
					c.commit();
				}else {
					c.rollback();
					return false;
				}
			}else {
				state = true;
				c.commit();
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				ps3.close();
				ps2.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public boolean removeUser(String id, String photoId) {
		String sql = "DELETE FROM user WHERE idu=?";
		String sql2 = "DELETE FROM account WHERE idu=?";
		String sql3 = "DELETE FROM userPhoto WHERE id=?";
		
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;

		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			ps = c.prepareStatement(sql);
			ps2= c.prepareStatement(sql2);
			ps3= c.prepareStatement(sql3);
			ps.setString(1, id);
			ps2.setString(1, id);
			
			if(ps2.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			if(ps.executeUpdate()!=1){
				c.rollback();
				return false;
			}
			
			if(photoId!=null){
				ps3.setString(1, photoId);
				if(ps3.executeUpdate()==1){
					state = true;
					c.commit();
				}else {
					c.rollback();
					return false;
				}
			}else {
				state = true;
				c.commit();
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				ps3.close();
				ps2.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	public void deleteHistoryMessage() {
		String sql = "truncate history";
		Connection c = null;
		Statement stmt = null;

		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			stmt.executeUpdate(sql);
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
	}
	
	public void checkBorder(String id) {
		String sql = "SELECT border,quantity FROM item WHERE id=?";
		String sql2 = "SELECT * FROM warning WHERE idi=? AND state=0";
		String sql3 = "INSERT INTO warning VALUES (null, false, ?, ?, ?)";
		String sql4 = "SHOW TABLE STATUS WHERE `Name` =  'warning'";
		Connection c = null;
		PreparedStatement ps = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		Statement stmt = null;
		ResultSet res = null;
		ResultSet res2 = null;
		ResultSet res3 = null;
	
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps2 = c.prepareStatement(sql2);
			ps3 = c.prepareStatement(sql3);
			stmt = c.createStatement();
			ps.setString(1, id);
			ps2.setString(1, id);
			ps3.setTimestamp(1, new Timestamp(new Date().getTime()));
			ps3.setString(2, "Položka s ID: " + id + " prolomila minimální hranici množství");
			ps3.setString(3, id);
			res = ps.executeQuery();
			res2 = ps2.executeQuery();
			res3 = stmt.executeQuery(sql4);
			
			while(res.next()){
				int a = res.getInt("border");
				int b = res.getInt("quantity");
				if(a > b) {
					if(!res2.next()){
						if(ps3.executeUpdate()!=1){
							message.addMessage(FacesMessage.SEVERITY_WARN, "Varovná zpáva neuložena");
						}else {
							while(res3.next()){
								addMessageToHistory("0", "", user.getId(), user.getFirstname() + " " + user.getLastname(), "Vytvorena varovná zpáva", "ID: " + (res3.getInt("Auto_increment")));
							}
						}
					}
				}else {
					return;
				}
			}
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res3.close();
				res2.close();
				res.close();
				stmt.close();
				ps3.close();
				ps2.close();
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
	}
	
	public List<WarnMessage> listOfWarnMessage() {
		String sql = "SELECT * FROM warning WHERE state=0 ORDER BY time DESC";
		List<WarnMessage> messages = new ArrayList<>();
		Connection c = null;
		Statement stmt = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			res = stmt.executeQuery(sql);
			
			while(res.next()){
				WarnMessage message = new WarnMessage();
				message.setId(res.getString("id"));
				message.setState(res.getBoolean("state"));
				message.setDate(res.getTimestamp("time"));
				message.setMessage(res.getString("message"));
				messages.add(message);
			}
			
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return messages;
	}
	
	public boolean warnMessage() {
		String sql = "SELECT id FROM warning WHERE state=0";
		boolean warn = false;
		Connection c = null;
		Statement stmt = null;
		ResultSet res = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			res = stmt.executeQuery(sql);
			
			while(res.next()){
				warn = true;
			}
			
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return warn;
	}
	
	public List<HistoryMessage> listOfHistoryMessage(String searchedHistoryString) {
		String sql = "SELECT * FROM history ORDER BY time DESC";
		String sql2 = "SELECT * FROM history WHERE itemId=? OR userId=? ORDER BY time DESC";
		String sql3 = "SELECT * FROM history WHERE itemName=? OR userName=? OR operation=? ORDER BY time DESC";
		boolean isName = false;
		List<HistoryMessage> messages = new ArrayList<>();
		Connection c = null;
		Statement stmt = null;
		PreparedStatement ps = null;
		ResultSet res = null;
		
		if(searchedHistoryString==null){
			searchedHistoryString = "";
		}
		
		char[] charArray = searchedHistoryString.toCharArray();
		for(char a : charArray){
			if(!Character.isDigit(a)){
				isName = true;
			}
		}
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			stmt = c.createStatement();
			if(isName){
				ps = c.prepareStatement(sql3);
				ps.setString(3, searchedHistoryString);
			}else {
				ps = c.prepareStatement(sql2);
			}
			ps.setString(1, searchedHistoryString);
			ps.setString(2, searchedHistoryString);
			
			if(searchedHistoryString.length()<1){
				res = stmt.executeQuery(sql);
			}else {
				res = ps.executeQuery();
			}
			while(res.next()){
				HistoryMessage message = new HistoryMessage();
				message.setItemId(res.getString("itemId"));
				message.setItemName(res.getString("itemName"));
				message.setUserId(res.getString("userId"));
				message.setUserName(res.getString("userName"));
				message.setDate(res.getTimestamp("time"));
				message.setOperation(res.getString("operation"));
				message.setMessage(res.getString("message"));
				messages.add(message);
			}
			
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				res.close();
				ps.close();
				stmt.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return messages;
	}
	
	public boolean updateCheckedWarnMessage(List<WarnMessage> selectedList) {
		String sql = "UPDATE warning SET state=1 WHERE id=?";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			c.setAutoCommit(false);
			ps = c.prepareStatement(sql);
			
			for(WarnMessage m : selectedList){
				ps.setString(1, m.getId());
				if(ps.executeUpdate()!=1){
					c.rollback();
					return false;
				}
			}
			state = true;
			c.commit();
			
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}

	public boolean addMessageToHistory(String itemId, String itemName, String userId, String userName, String operation, String message) {
		String sql = "INSERT INTO history(itemId,itemName,userId,userName,time,operation,message) VALUES (?,?,?,?,?,?,?)";
		boolean state = false;
		Connection c = null;
		PreparedStatement ps = null;
		
		try {
			Class.forName("com.mysql.jdbc.Driver");
			c = DriverManager.getConnection(url, USERNAME, PASSWORD);
			ps = c.prepareStatement(sql);
			ps.setInt(1, Integer.parseInt(itemId));
			ps.setString(2, itemName);
			ps.setInt(3, Integer.parseInt(userId));
			ps.setString(4, userName);
			ps.setTimestamp(5, new Timestamp(new Date().getTime()));
			ps.setString(6, operation);
			ps.setString(7, message);
			
			if(ps.executeUpdate()==1){
				state = true;
			}
			
			
		} catch (ClassNotFoundException e) {
			e.getMessage();
		} catch (SQLException e) {
			e.getMessage();
		} finally {
			try {
				ps.close();
				c.close();
			} catch (SQLException e) {
				e.getMessage();
			}
		}
		return state;
	}
	
	
	//------------------------------------setters/getters--------------------------------------
	
	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public Account getAccount() {
		return account;
	}

	public void setAccount(Account account) {
		this.account = account;
	}

	public static String getUsername() {
		return USERNAME;
	}

	public static String getPassword() {
		return PASSWORD;
	}

	public static String getHost() {
		return HOST;
	}

	public static String getPort() {
		return PORT;
	}

	public static String getDatabaseName() {
		return DATABASE_NAME;
	}

	public static String getUrl() {
		return url;
	}

	public Message getMessage() {
		return message;
	}

	public void setMessage(Message message) {
		this.message = message;
	}

	public Item getItem() {
		return item;
	}

	public void setItem(Item item) {
		this.item = item;
	}

	public LoadedUser getLoadedUser() {
		return loadedUser;
	}

	public void setLoadedUser(LoadedUser loadedUser) {
		this.loadedUser = loadedUser;
	}
}
